@@ -21,34 +21,59 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @file file_collection.cc
  */
 
-#include "config.h"
+#include <unordered_map>
+
+#include "file_collection.hh"
 
 #include <glob.h>
 
-#include <unordered_map>
-
-#include "base/opt_util.hh"
 #include "base/humanize.network.hh"
 #include "base/isc.hh"
-#include "logfile.hh"
-#include "file_collection.hh"
-#include "pcrepp/pcrepp.hh"
-#include "tailer/tailer.looper.hh"
-#include "service_tags.hh"
+#include "base/itertools.hh"
+#include "base/opt_util.hh"
+#include "base/string_util.hh"
+#include "config.h"
 #include "lnav_util.hh"
+#include "logfile.hh"
+#include "pcap_manager.hh"
+#include "service_tags.hh"
+#include "tailer/tailer.looper.hh"
 
 static std::mutex REALPATH_CACHE_MUTEX;
 static std::unordered_map<std::string, std::string> REALPATH_CACHE;
 
-void file_collection::close_files(const std::vector<std::shared_ptr<logfile>> &files)
+child_poll_result_t
+child_poller::poll(file_collection& fc)
+{
+    if (!this->cp_child) {
+        return child_poll_result_t::FINISHED;
+    }
+
+    auto poll_res = std::move(this->cp_child.value()).poll();
+    this->cp_child = nonstd::nullopt;
+    return poll_res.match(
+        [this](auto_pid<process_state::running>& alive) {
+            this->cp_child = std::move(alive);
+            return child_poll_result_t::ALIVE;
+        },
+        [this, &fc](auto_pid<process_state::finished>& finished) {
+            require(this->cp_finalizer);
+
+            this->cp_finalizer(fc, finished);
+            return child_poll_result_t::FINISHED;
+        });
+}
+
+void
+file_collection::close_files(const std::vector<std::shared_ptr<logfile>>& files)
 {
     for (const auto& lf : files) {
         auto actual_path_opt = lf->get_actual_path();
@@ -58,7 +83,8 @@ void file_collection::close_files(const std::vector<std::shared_ptr<logfile>> &f
             auto path_str = actual_path_opt.value().string();
 
             for (auto iter = REALPATH_CACHE.begin();
-                 iter != REALPATH_CACHE.end();) {
+                 iter != REALPATH_CACHE.end();)
+            {
                 if (iter->first == path_str || iter->second == path_str) {
                     iter = REALPATH_CACHE.erase(iter);
                 } else {
@@ -68,9 +94,7 @@ void file_collection::close_files(const std::vector<std::shared_ptr<logfile>> &f
         } else {
             this->fc_file_names.erase(lf->get_filename());
         }
-        auto file_iter = find(this->fc_files.begin(),
-                              this->fc_files.end(),
-                              lf);
+        auto file_iter = find(this->fc_files.begin(), this->fc_files.end(), lf);
         if (file_iter != this->fc_files.end()) {
             this->fc_files.erase(file_iter);
         }
@@ -80,43 +104,45 @@ void file_collection::close_files(const std::vector<std::shared_ptr<logfile>> &f
     this->regenerate_unique_file_names();
 }
 
-void file_collection::regenerate_unique_file_names()
+void
+file_collection::regenerate_unique_file_names()
 {
     unique_path_generator upg;
 
-    for (const auto &lf : this->fc_files) {
+    for (const auto& lf : this->fc_files) {
         upg.add_source(lf);
     }
 
     upg.generate();
 
     this->fc_largest_path_length = 0;
-    for (const auto &pair : this->fc_name_to_errors) {
+    for (const auto& pair : this->fc_name_to_errors) {
         auto path = ghc::filesystem::path(pair.first).filename().string();
 
         if (path.length() > this->fc_largest_path_length) {
             this->fc_largest_path_length = path.length();
         }
     }
-    for (const auto &lf : this->fc_files) {
-        const auto &path = lf->get_unique_path();
+    for (const auto& lf : this->fc_files) {
+        const auto& path = lf->get_unique_path();
 
         if (path.length() > this->fc_largest_path_length) {
             this->fc_largest_path_length = path.length();
         }
     }
-    for (const auto &pair : this->fc_other_files) {
+    for (const auto& pair : this->fc_other_files) {
         switch (pair.second.ofd_format) {
-            case file_format_t::FF_UNKNOWN:
-            case file_format_t::FF_ARCHIVE:
-            case file_format_t::FF_SQLITE_DB: {
+            case file_format_t::UNKNOWN:
+            case file_format_t::ARCHIVE:
+            case file_format_t::PCAP:
+            case file_format_t::SQLITE_DB: {
                 auto bn = ghc::filesystem::path(pair.first).filename().string();
                 if (bn.length() > this->fc_largest_path_length) {
                     this->fc_largest_path_length = bn.length();
                 }
                 break;
             }
-            case file_format_t::FF_REMOTE: {
+            case file_format_t::REMOTE: {
                 if (pair.first.length() > this->fc_largest_path_length) {
                     this->fc_largest_path_length = pair.first.length();
                 }
@@ -126,7 +152,8 @@ void file_collection::regenerate_unique_file_names()
     }
 }
 
-void file_collection::merge(const file_collection &other)
+void
+file_collection::merge(file_collection& other)
 {
     this->fc_recursive = this->fc_recursive || other.fc_recursive;
     this->fc_rotated = this->fc_rotated || other.fc_rotated;
@@ -135,31 +162,38 @@ void file_collection::merge(const file_collection &other)
                                  other.fc_synced_files.end());
     this->fc_name_to_errors.insert(other.fc_name_to_errors.begin(),
                                    other.fc_name_to_errors.end());
-    this->fc_file_names.insert(other.fc_file_names.begin(),
-                               other.fc_file_names.end());
+    this->fc_file_names.insert(
+        std::make_move_iterator(other.fc_file_names.begin()),
+        std::make_move_iterator(other.fc_file_names.end()));
     if (!other.fc_files.empty()) {
         for (const auto& lf : other.fc_files) {
             this->fc_name_to_errors.erase(lf->get_filename());
         }
-        this->fc_files.insert(this->fc_files.end(),
-                              other.fc_files.begin(),
-                              other.fc_files.end());
+        this->fc_files.insert(
+            this->fc_files.end(), other.fc_files.begin(), other.fc_files.end());
         this->fc_files_generation += 1;
     }
-    for (auto &pair : other.fc_renamed_files) {
+    for (auto& pair : other.fc_renamed_files) {
         pair.first->set_filename(pair.second);
     }
     this->fc_closed_files.insert(other.fc_closed_files.begin(),
                                  other.fc_closed_files.end());
     this->fc_other_files.insert(other.fc_other_files.begin(),
                                 other.fc_other_files.end());
+    if (!other.fc_child_pollers.empty()) {
+        this->fc_child_pollers.insert(
+            this->fc_child_pollers.begin(),
+            std::make_move_iterator(other.fc_child_pollers.begin()),
+            std::make_move_iterator(other.fc_child_pollers.end()));
+        other.fc_child_pollers.clear();
+    }
 }
 
 /**
  * Functor used to compare files based on their device and inode number.
  */
 struct same_file {
-    explicit same_file(const struct stat &stat) : sf_stat(stat) {};
+    explicit same_file(const struct stat& stat) : sf_stat(stat){};
 
     /**
      * Compare the given log file against the 'stat' given in the constructor.
@@ -167,14 +201,13 @@ struct same_file {
      * @return    True if the dev/inode values in the stat given in the
      *   constructor matches the stat in the logfile object.
      */
-    bool operator()(const std::shared_ptr<logfile> &lf) const
+    bool operator()(const std::shared_ptr<logfile>& lf) const
     {
-        return !lf->is_closed() &&
-               this->sf_stat.st_dev == lf->get_stat().st_dev &&
-               this->sf_stat.st_ino == lf->get_stat().st_ino;
-    };
+        return !lf->is_closed() && this->sf_stat.st_dev == lf->get_stat().st_dev
+            && this->sf_stat.st_ino == lf->get_stat().st_ino;
+    }
 
-    const struct stat &sf_stat;
+    const struct stat& sf_stat;
 };
 
 /**
@@ -187,19 +220,29 @@ struct same_file {
  * @param required Specifies whether or not the file must exist and be valid.
  */
 std::future<file_collection>
-file_collection::watch_logfile(const std::string &filename,
-                               logfile_open_options &loo, bool required)
+file_collection::watch_logfile(const std::string& filename,
+                               logfile_open_options& loo,
+                               bool required)
 {
     file_collection retval;
     struct stat st;
     int rc;
 
     if (this->fc_closed_files.count(filename)) {
-        return lnav::futures::make_ready_future(retval);
+        return lnav::futures::make_ready_future(std::move(retval));
     }
 
     if (loo.loo_fd != -1) {
         rc = fstat(loo.loo_fd, &st);
+        if (rc == 0) {
+            loo.with_stat_for_temp(st);
+        }
+    } else if (loo.loo_temp_file) {
+        memset(&st, 0, sizeof(st));
+        st.st_dev = loo.loo_temp_dev;
+        st.st_ino = loo.loo_temp_ino;
+        st.st_mode = S_IFREG;
+        rc = 0;
     } else {
         rc = stat(filename.c_str(), &st);
     }
@@ -208,18 +251,18 @@ file_collection::watch_logfile(const std::string &filename,
         if (S_ISDIR(st.st_mode) && this->fc_recursive) {
             std::string wilddir = filename + "/*";
 
-            if (this->fc_file_names.find(wilddir) ==
-                this->fc_file_names.end()) {
+            if (this->fc_file_names.find(wilddir) == this->fc_file_names.end())
+            {
                 retval.fc_file_names.emplace(wilddir, logfile_open_options());
             }
-            return lnav::futures::make_ready_future(retval);
+            return lnav::futures::make_ready_future(std::move(retval));
         }
         if (!S_ISREG(st.st_mode)) {
             if (required) {
                 rc = -1;
                 errno = EINVAL;
             } else {
-                return lnav::futures::make_ready_future(retval);
+                return lnav::futures::make_ready_future(std::move(retval));
             }
         }
         auto err_iter = this->fc_name_to_errors.find(filename);
@@ -231,37 +274,41 @@ file_collection::watch_logfile(const std::string &filename,
     }
     if (rc == -1) {
         if (required) {
-            retval.fc_name_to_errors.emplace(filename, file_error_info{
-                time(nullptr),
-                std::string(strerror(errno)),
-            });
+            retval.fc_name_to_errors.emplace(filename,
+                                             file_error_info{
+                                                 time(nullptr),
+                                                 std::string(strerror(errno)),
+                                             });
         }
-        return lnav::futures::make_ready_future(retval);
+        return lnav::futures::make_ready_future(std::move(retval));
     }
 
-    auto stat_iter = find_if(this->fc_new_stats.begin(),
-                             this->fc_new_stats.end(),
-                             [&st](auto& elem) {
-                                 return st.st_ino == elem.st_ino &&
-                                        st.st_dev == elem.st_dev;
-                             });
-    if (stat_iter != this->fc_new_stats.end()) {
+    if (this->fc_new_stats | lnav::itertools::find_if([&st](const auto& elem) {
+            return st.st_ino == elem.st_ino && st.st_dev == elem.st_dev;
+        }))
+    {
         // this file is probably a link that we have already scanned in this
         // pass.
-        return lnav::futures::make_ready_future(retval);
+        return lnav::futures::make_ready_future(std::move(retval));
     }
 
     this->fc_new_stats.emplace_back(st);
-    auto file_iter = find_if(this->fc_files.begin(),
-                             this->fc_files.end(),
-                             same_file(st));
+
+    auto file_iter = std::find_if(
+        this->fc_files.begin(), this->fc_files.end(), same_file(st));
 
     if (file_iter == this->fc_files.end()) {
         if (this->fc_other_files.find(filename) != this->fc_other_files.end()) {
-            return lnav::futures::make_ready_future(retval);
+            return lnav::futures::make_ready_future(std::move(retval));
         }
 
-        auto func = [filename, st, loo, prog = this->fc_progress, errs = this->fc_name_to_errors]() mutable {
+        require(this->fc_progress.get() != nullptr);
+
+        auto func = [filename,
+                     st,
+                     loo2 = std::move(loo),
+                     prog = this->fc_progress,
+                     errs = this->fc_name_to_errors]() mutable {
             file_collection retval;
 
             if (errs.find(filename) != errs.end()) {
@@ -269,42 +316,96 @@ file_collection::watch_logfile(const std::string &filename,
                 return retval;
             }
 
-            auto ff = detect_file_format(filename);
+            auto ff = loo2.loo_temp_file ? file_format_t::UNKNOWN
+                                         : detect_file_format(filename);
 
+            loo2.loo_file_format = ff;
             switch (ff) {
-                case file_format_t::FF_SQLITE_DB:
+                case file_format_t::SQLITE_DB:
                     retval.fc_other_files[filename].ofd_format = ff;
                     break;
 
-                case file_format_t::FF_ARCHIVE: {
-                    nonstd::optional<std::list<archive_manager::extract_progress>::iterator>
+                case file_format_t::PCAP: {
+                    auto res = pcap_manager::convert(filename);
+
+                    if (res.isOk()) {
+                        auto convert_res = res.unwrap();
+
+                        loo2.with_fd(std::move(convert_res.cr_destination));
+                        retval.fc_child_pollers.emplace_back(child_poller{
+                            std::move(convert_res.cr_child),
+                            [filename,
+                             st,
+                             error_queue = convert_res.cr_error_queue](
+                                auto& fc, auto& child) {
+                                if (child.was_normal_exit()
+                                    && child.exit_status() == EXIT_SUCCESS)
+                                {
+                                    log_info("pcap[%d] exited normally",
+                                             child.in());
+                                    return;
+                                }
+                                log_error("pcap[%d] exited with %d",
+                                          child.in(),
+                                          child.status());
+                                fc.fc_name_to_errors.emplace(
+                                    filename,
+                                    file_error_info{
+                                        st.st_mtime,
+                                        fmt::format(
+                                            FMT_STRING("{}"),
+                                            fmt::join(*error_queue, "\n")),
+                                    });
+                            },
+                        });
+                        auto open_res = logfile::open(filename, loo2);
+                        if (open_res.isOk()) {
+                            retval.fc_files.push_back(open_res.unwrap());
+                        } else {
+                            retval.fc_name_to_errors.emplace(
+                                filename,
+                                file_error_info{
+                                    st.st_mtime,
+                                    open_res.unwrapErr(),
+                                });
+                        }
+                    } else {
+                        retval.fc_name_to_errors.emplace(filename,
+                                                         file_error_info{
+                                                             st.st_mtime,
+                                                             res.unwrapErr(),
+                                                         });
+                    }
+                    break;
+                }
+
+                case file_format_t::ARCHIVE: {
+                    nonstd::optional<
+                        std::list<archive_manager::extract_progress>::iterator>
                         prog_iter_opt;
 
-                    if (loo.loo_source == logfile_name_source::ARCHIVE) {
+                    if (loo2.loo_source == logfile_name_source::ARCHIVE) {
                         // Don't try to open nested archives
                         return retval;
                     }
 
                     auto res = archive_manager::walk_archive_files(
                         filename,
-                        [prog, &prog_iter_opt](
-                            const auto &path,
-                            const auto total) {
+                        [prog, &prog_iter_opt](const auto& path,
+                                               const auto total) {
                             safe::WriteAccess<safe_scan_progress> sp(*prog);
 
                             prog_iter_opt | [&sp](auto prog_iter) {
                                 sp->sp_extractions.erase(prog_iter);
                             };
                             auto prog_iter = sp->sp_extractions.emplace(
-                                sp->sp_extractions.begin(),
-                                path, total);
+                                sp->sp_extractions.begin(), path, total);
                             prog_iter_opt = prog_iter;
 
                             return &(*prog_iter);
                         },
-                        [&filename, &retval](
-                            const auto &tmp_path,
-                            const auto &entry) {
+                        [&filename, &retval](const auto& tmp_path,
+                                             const auto& entry) {
                             auto arc_path = ghc::filesystem::relative(
                                 entry.path(), tmp_path);
                             auto custom_name = filename / arc_path;
@@ -330,18 +431,16 @@ file_collection::watch_logfile(const std::string &filename,
                         log_error("archive extraction failed: %s",
                                   res.unwrapErr().c_str());
                         retval.clear();
-                        retval.fc_name_to_errors.emplace(
-                            filename,
-                            file_error_info{
-                                st.st_mtime,
-                                res.unwrapErr(),
-                            });
+                        retval.fc_name_to_errors.emplace(filename,
+                                                         file_error_info{
+                                                             st.st_mtime,
+                                                             res.unwrapErr(),
+                                                         });
                     } else {
                         retval.fc_other_files[filename] = ff;
                     }
                     {
-                        prog_iter_opt |
-                        [&prog](auto prog_iter) {
+                        prog_iter_opt | [&prog](auto prog_iter) {
                             prog->writeAccess()->sp_extractions.erase(
                                 prog_iter);
                         };
@@ -352,13 +451,13 @@ file_collection::watch_logfile(const std::string &filename,
                 default:
                     log_info("loading new file: filename=%s", filename.c_str());
 
-                    auto open_res = logfile::open(filename, loo);
+                    auto open_res = logfile::open(filename, loo2);
                     if (open_res.isOk()) {
                         retval.fc_files.push_back(open_res.unwrap());
-                    }
-                    else {
+                    } else {
                         retval.fc_name_to_errors.emplace(
-                            filename, file_error_info{
+                            filename,
+                            file_error_info{
                                 st.st_mtime,
                                 open_res.unwrapErr(),
                             });
@@ -369,19 +468,19 @@ file_collection::watch_logfile(const std::string &filename,
             return retval;
         };
 
-        return std::async(std::launch::async, func);
-    } else {
-        auto lf = *file_iter;
-
-        if (lf->is_valid_filename() && lf->get_filename() != filename) {
-            /* The file is already loaded, but has been found under a different
-             * name.  We just need to update the stored file name.
-             */
-            retval.fc_renamed_files.emplace_back(lf, filename);
-        }
+        return std::async(std::launch::async, std::move(func));
     }
 
-    return lnav::futures::make_ready_future(retval);
+    auto lf = *file_iter;
+
+    if (lf->is_valid_filename() && lf->get_filename() != filename) {
+        /* The file is already loaded, but has been found under a different
+         * name.  We just need to update the stored file name.
+         */
+        retval.fc_renamed_files.emplace_back(lf, filename);
+    }
+
+    return lnav::futures::make_ready_future(std::move(retval));
 }
 
 /**
@@ -390,10 +489,12 @@ file_collection::watch_logfile(const std::string &filename,
  * @param path     The glob pattern to expand.
  * @param required Passed to watch_logfile.
  */
-void file_collection::expand_filename(lnav::futures::future_queue<file_collection> &fq,
-                                      const std::string &path,
-                                      logfile_open_options &loo,
-                                      bool required)
+void
+file_collection::expand_filename(
+    lnav::futures::future_queue<file_collection>& fq,
+    const std::string& path,
+    logfile_open_options& loo,
+    bool required)
 {
     static_root_mem<glob_t, globfree> gl;
 
@@ -428,27 +529,29 @@ void file_collection::expand_filename(lnav::futures::future_queue<file_collectio
                     }
 
                     file_collection retval;
+                    logfile_open_options_base loo_base{loo};
 
-                    isc::to<tailer::looper &, services::remote_tailer_t>()
-                        .send([=](auto &tlooper) {
-                            tlooper.add_remote(rp, loo);
+                    isc::to<tailer::looper&, services::remote_tailer_t>().send(
+                        [rp, loo_base](auto& tlooper) {
+                            tlooper.add_remote(rp, loo_base);
                         });
-                    retval.fc_other_files[path] = file_format_t::FF_REMOTE;
+                    retval.fc_other_files[path] = file_format_t::REMOTE;
                     {
-                        this->fc_progress->writeAccess()->
-                            sp_tailers[fmt::format("{}", rp.home())].tp_message =
-                            "Initializing...";
+                        this->fc_progress->writeAccess()
+                            ->sp_tailers[fmt::to_string(rp.home())]
+                            .tp_message
+                            = "Initializing...";
                     }
 
-                    fq.push_back(lnav::futures::make_ready_future(retval));
+                    fq.push_back(
+                        lnav::futures::make_ready_future(std::move(retval)));
                     return;
                 }
 
                 required = false;
             }
         }
-        if (gl->gl_pathc > 1 ||
-            strcmp(path.c_str(), gl->gl_pathv[0]) != 0) {
+        if (gl->gl_pathc > 1 || strcmp(path.c_str(), gl->gl_pathv[0]) != 0) {
             required = false;
         }
 
@@ -460,13 +563,15 @@ void file_collection::expand_filename(lnav::futures::future_queue<file_collectio
             if (iter == REALPATH_CACHE.end()) {
                 auto_mem<char> abspath;
 
-                if ((abspath = realpath(gl->gl_pathv[lpc], nullptr)) ==
-                    nullptr) {
-                    auto errmsg = strerror(errno);
+                if ((abspath = realpath(gl->gl_pathv[lpc], nullptr)) == nullptr)
+                {
+                    auto* errmsg = strerror(errno);
 
                     if (required) {
-                        fprintf(stderr, "Cannot find file: %s -- %s",
-                                gl->gl_pathv[lpc], errmsg);
+                        fprintf(stderr,
+                                "Cannot find file: %s -- %s",
+                                gl->gl_pathv[lpc],
+                                errmsg);
                     } else if (loo.loo_source != logfile_name_source::REMOTE) {
                         // XXX The remote code path adds the file name before
                         // the file exists...  not sure checking for that here
@@ -474,26 +579,27 @@ void file_collection::expand_filename(lnav::futures::future_queue<file_collectio
                         file_collection retval;
 
                         if (gl->gl_pathc == 1) {
-                            retval.fc_name_to_errors.emplace(
-                                path, file_error_info{
-                                    time(nullptr),
-                                    errmsg,
-                                });
+                            retval.fc_name_to_errors.emplace(path,
+                                                             file_error_info{
+                                                                 time(nullptr),
+                                                                 errmsg,
+                                                             });
                         } else {
-                            retval.fc_name_to_errors.emplace(
-                                path_str, file_error_info{
-                                    time(nullptr),
-                                    errmsg,
-                                });
+                            retval.fc_name_to_errors.emplace(path_str,
+                                                             file_error_info{
+                                                                 time(nullptr),
+                                                                 errmsg,
+                                                             });
                         }
-                        fq.push_back(lnav::futures::make_ready_future(retval));
+                        fq.push_back(lnav::futures::make_ready_future(
+                            std::move(retval)));
                     }
                     continue;
-                } else {
-                    auto p = REALPATH_CACHE.emplace(path_str, abspath.in());
-
-                    iter = p.first;
                 }
+
+                auto p = REALPATH_CACHE.emplace(path_str, abspath.in());
+
+                iter = p.first;
             }
 
             if (required || access(iter->second.c_str(), R_OK) == 0) {
@@ -503,22 +609,22 @@ void file_collection::expand_filename(lnav::futures::future_queue<file_collectio
     }
 }
 
-file_collection file_collection::rescan_files(bool required)
+file_collection
+file_collection::rescan_files(bool required)
 {
     file_collection retval;
-    lnav::futures::future_queue<file_collection> fq([&retval](auto &fc) {
-        retval.merge(fc);
-    });
+    lnav::futures::future_queue<file_collection> fq(
+        [&retval](auto& fc) { retval.merge(fc); });
 
-    for (auto &pair : this->fc_file_names) {
-        if (pair.second.loo_fd == -1) {
+    for (auto& pair : this->fc_file_names) {
+        if (!pair.second.loo_temp_file) {
             this->expand_filename(fq, pair.first, pair.second, required);
             if (this->fc_rotated) {
                 std::string path = pair.first + ".*";
 
                 this->expand_filename(fq, path, pair.second, false);
             }
-        } else {
+        } else if (pair.second.loo_fd.get() != -1) {
             fq.push_back(watch_logfile(pair.first, pair.second, required));
         }
 
@@ -534,3 +640,10 @@ file_collection file_collection::rescan_files(bool required)
 
     return retval;
 }
+
+void
+file_collection::request_close(const std::shared_ptr<logfile>& lf)
+{
+    lf->close();
+    this->fc_files_generation += 1;
+}
