@@ -21,24 +21,25 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @file log_accel.cc
  */
 
-#include "config.h"
-
 #include <algorithm>
 
 #include "log_accel.hh"
 
+#include "config.h"
+
 const double log_accel::MIN_RANGE = 5.0;
 const double log_accel::THRESHOLD = 0.1;
 
-bool log_accel::add_point(int64_t point)
+bool
+log_accel::add_point(int64_t point)
 {
     require(this->la_velocity_size < HISTORY_SIZE);
 
@@ -48,14 +49,14 @@ bool log_accel::add_point(int64_t point)
         // require(this->la_last_point >= point);
 
         // Compute the message velocity.
-        this->la_velocity[this->la_velocity_size] = (
-            this->la_last_point - point);
+        this->la_velocity[this->la_velocity_size]
+            = (this->la_last_point - point);
 
         // Find the range of velocities so we can normalize.
-        this->la_min_velocity = std::min(this->la_min_velocity,
-                                         this->la_velocity[this->la_velocity_size]);
-        this->la_max_velocity = std::max(this->la_max_velocity,
-                                         this->la_velocity[this->la_velocity_size]);
+        this->la_min_velocity = std::min(
+            this->la_min_velocity, this->la_velocity[this->la_velocity_size]);
+        this->la_max_velocity = std::max(
+            this->la_max_velocity, this->la_velocity[this->la_velocity_size]);
 
         this->la_velocity_size += 1;
     }
@@ -66,7 +67,8 @@ bool log_accel::add_point(int64_t point)
     return this->la_velocity_size < HISTORY_SIZE;
 }
 
-double log_accel::get_avg_accel() const
+double
+log_accel::get_avg_accel() const
 {
     double avg_accel = 0, total_accel = 0;
     // Compute the range of values so we can normalize.
@@ -74,8 +76,8 @@ double log_accel::get_avg_accel() const
 
     range = std::max(range, MIN_RANGE);
     for (int lpc = 0; lpc < (this->la_velocity_size - 1); lpc++) {
-        double accel =
-            (this->la_velocity[lpc] - this->la_velocity[lpc + 1]) / range;
+        double accel
+            = (this->la_velocity[lpc] - this->la_velocity[lpc + 1]) / range;
         total_accel += accel;
     }
 
@@ -86,7 +88,8 @@ double log_accel::get_avg_accel() const
     return avg_accel;
 }
 
-log_accel::direction_t log_accel::get_direction() const
+log_accel::direction_t
+log_accel::get_direction() const
 {
     double avg_accel = this->get_avg_accel();
     direction_t retval;
