@@ -4,18 +4,21 @@
 // https://github.com/martinmoene/optional-lite
 //
 // Distributed under the Boost Software License, Version 1.0.
-// (See accompanying file LICENSE.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+// (See accompanying file LICENSE.txt or copy at
+// http://www.boost.org/LICENSE_1_0.txt)
 
 #pragma once
 
 #ifndef NONSTD_OPTIONAL_LITE_HPP
-#define NONSTD_OPTIONAL_LITE_HPP
+#    define NONSTD_OPTIONAL_LITE_HPP
 
-#define optional_lite_MAJOR  3
-#define optional_lite_MINOR  4
-#define optional_lite_PATCH  0
+#    define optional_lite_MAJOR 3
+#    define optional_lite_MINOR 4
+#    define optional_lite_PATCH 0
 
-#define optional_lite_VERSION  optional_STRINGIFY(optional_lite_MAJOR) "." optional_STRINGIFY(optional_lite_MINOR) "." optional_STRINGIFY(optional_lite_PATCH)
+#    define optional_lite_VERSION \
+        optional_STRINGIFY(optional_lite_MAJOR) "." optional_STRINGIFY( \
+            optional_lite_MINOR) "." optional_STRINGIFY(optional_lite_PATCH)
 
 #define optional_STRINGIFY(  x )  optional_STRINGIFY_( x )
 #define optional_STRINGIFY_( x )  #x
@@ -1483,6 +1486,9 @@ public:
         has_value_ = false;
     }
 
+    template<typename F>
+    auto map(F func) -> optional<decltype(func(this->value()))>;
+
 private:
     void this_type_does_not_support_comparisons() const {}
 
@@ -1744,6 +1750,17 @@ optional<T> make_optional( T const & value )
 
 #endif // optional_CPP11_OR_GREATER
 
+template<typename T>
+template<typename F>
+auto optional<T>::map(F func) -> optional<decltype(func(this->value()))>
+{
+    if (this->has_value()) {
+        return make_optional(func(this->value()));
+    }
+
+    return nullopt;
+}
+
 } // namespace optional_lite
 
 using optional_lite::optional;
