@@ -21,76 +21,191 @@
  * DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY
  * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "config.h"
-
-#include <assert.h>
 #include <unordered_set>
 
-#include "base/time_util.hh"
-#include "pcrepp/pcrepp.hh"
 #include "relative_time.hh"
 
-using namespace std;
+#include "base/time_util.hh"
+#include "config.h"
+#include "pcrepp/pcre2pp.hh"
+#include "scn/scn.h"
+
 using namespace std::chrono_literals;
 
 static const struct {
-    const char *name;
-    pcrepp pcre;
+    const char* name;
+    lnav::pcre2pp::code pcre;
 } MATCHERS[relative_time::RTT__MAX] = {
-    { "ws", pcrepp("\\A\\s+\\b") },
-    { "am", pcrepp("\\Aam|a\\.m\\.\\b") },
-    { "pm", pcrepp("\\Apm|p\\.m\\.\\b") },
-    { "a", pcrepp("\\Aa\\b") },
-    { "an", pcrepp("\\Aan\\b") },
-    { "at", pcrepp("\\Aat\\b") },
-    { "time",
-        pcrepp("\\A(\\d{1,2}):(\\d{2})(?::(\\d{2})(?:\\.(\\d{3,6}))?)?") },
-    { "num", pcrepp("\\A((?:-|\\+)?\\d+)") },
+    {
+        "ws",
+        lnav::pcre2pp::code::from_const("\\A\\s+\\b"),
+    },
+    {
+        "am",
+        lnav::pcre2pp::code::from_const("\\Aam|a\\.m\\.\\b"),
+    },
+    {
+        "pm",
+        lnav::pcre2pp::code::from_const("\\Apm|p\\.m\\.\\b"),
+    },
+    {
+        "a",
+        lnav::pcre2pp::code::from_const("\\Aa\\b"),
+    },
+    {
+        "an",
+        lnav::pcre2pp::code::from_const("\\Aan\\b"),
+    },
+    {
+        "at",
+        lnav::pcre2pp::code::from_const("\\Aat\\b"),
+    },
+    {
+        "time",
+        lnav::pcre2pp::code::from_const(
+            "\\A(\\d{1,2}):(\\d{2})(?::(\\d{2})(?:\\.(\\d{3,6}))?)?"),
+    },
+    {
+        "num",
+        lnav::pcre2pp::code::from_const("\\A((?:-|\\+)?\\d+)"),
+    },
 
-    { "sun", pcrepp("\\Asun(days?)?\\b") },
-    { "mon", pcrepp("\\Amon(days?)?\\b") },
-    { "tue", pcrepp("\\Atue(s(days?)?)?\\b") },
-    { "wed", pcrepp("\\Awed(nesdays?)?\\b") },
-    { "thu", pcrepp("\\Athu(rsdays?)?\\b") },
-    { "fri", pcrepp("\\Afri(days?)?\\b") },
-    { "sat", pcrepp("\\Asat(urdays?)?\\b") },
+    {
+        "sun",
+        lnav::pcre2pp::code::from_const("\\Asun(days?)?\\b"),
+    },
+    {
+        "mon",
+        lnav::pcre2pp::code::from_const("\\Amon(days?)?\\b"),
+    },
+    {
+        "tue",
+        lnav::pcre2pp::code::from_const("\\Atue(s(days?)?)?\\b"),
+    },
+    {
+        "wed",
+        lnav::pcre2pp::code::from_const("\\Awed(nesdays?)?\\b"),
+    },
+    {
+        "thu",
+        lnav::pcre2pp::code::from_const("\\Athu(rsdays?)?\\b"),
+    },
+    {
+        "fri",
+        lnav::pcre2pp::code::from_const("\\Afri(days?)?\\b"),
+    },
+    {
+        "sat",
+        lnav::pcre2pp::code::from_const("\\Asat(urdays?)?\\b"),
+    },
 
-    { "us", pcrepp("\\A(?:micros(?:econds?)?|us(?![a-zA-Z]))") },
-    { "ms", pcrepp("\\A(?:millis(?:econds?)?|ms(?![a-zA-Z]))") },
-    { "sec", pcrepp("\\As(?:ec(?:onds?)?)?(?![a-zA-Z])") },
-    { "min", pcrepp("\\Am(?:in(?:utes?)?)?(?![a-zA-Z])") },
-    { "h", pcrepp("\\Ah(?:ours?)?(?![a-zA-Z])") },
-    { "day", pcrepp("\\Ad(?:ays?)?(?![a-zA-Z])") },
-    { "week", pcrepp("\\Aw(?:eeks?)?(?![a-zA-Z])") },
-    { "mon", pcrepp("\\Amon(?:ths?)?(?![a-zA-Z])") },
-    { "year", pcrepp("\\Ay(?:ears?)?(?![a-zA-Z])") },
-    { "today", pcrepp("\\Atoday\\b") },
-    { "yest", pcrepp("\\Ayesterday\\b") },
-    { "tomo", pcrepp("\\Atomorrow\\b") },
-    { "noon", pcrepp("\\Anoon\\b") },
-    { "and", pcrepp("\\Aand\\b") },
-    { "the", pcrepp("\\Athe\\b") },
-    { "ago", pcrepp("\\Aago\\b") },
-    { "lter", pcrepp("\\Alater\\b") },
-    { "bfor", pcrepp("\\Abefore\\b") },
-    { "aft", pcrepp("\\Aafter\\b") },
-    { "now", pcrepp("\\Anow\\b") },
-    { "here", pcrepp("\\Ahere\\b") },
-    { "next", pcrepp("\\Anext\\b") },
-    { "previous", pcrepp("\\A(?:previous\\b|last\\b)") },
+    {
+        "us",
+        lnav::pcre2pp::code::from_const(
+            "\\A(?:micros(?:econds?)?|us(?![a-zA-Z]))"),
+    },
+    {
+        "ms",
+        lnav::pcre2pp::code::from_const(
+            "\\A(?:millis(?:econds?)?|ms(?![a-zA-Z]))"),
+    },
+    {
+        "sec",
+        lnav::pcre2pp::code::from_const("\\As(?:ec(?:onds?)?)?(?![a-zA-Z])"),
+    },
+    {
+        "min",
+        lnav::pcre2pp::code::from_const("\\Am(?:in(?:utes?)?)?(?![a-zA-Z])"),
+    },
+    {
+        "h",
+        lnav::pcre2pp::code::from_const("\\Ah(?:ours?)?(?![a-zA-Z])"),
+    },
+    {
+        "day",
+        lnav::pcre2pp::code::from_const("\\Ad(?:ays?)?(?![a-zA-Z])"),
+    },
+    {
+        "week",
+        lnav::pcre2pp::code::from_const("\\Aw(?:eeks?)?(?![a-zA-Z])"),
+    },
+    {
+        "mon",
+        lnav::pcre2pp::code::from_const("\\Amon(?:ths?)?(?![a-zA-Z])"),
+    },
+    {
+        "year",
+        lnav::pcre2pp::code::from_const("\\Ay(?:ears?)?(?![a-zA-Z])"),
+    },
+    {
+        "today",
+        lnav::pcre2pp::code::from_const("\\Atoday\\b"),
+    },
+    {
+        "yest",
+        lnav::pcre2pp::code::from_const("\\Ayesterday\\b"),
+    },
+    {
+        "tomo",
+        lnav::pcre2pp::code::from_const("\\Atomorrow\\b"),
+    },
+    {
+        "noon",
+        lnav::pcre2pp::code::from_const("\\Anoon\\b"),
+    },
+    {
+        "and",
+        lnav::pcre2pp::code::from_const("\\Aand\\b"),
+    },
+    {
+        "the",
+        lnav::pcre2pp::code::from_const("\\Athe\\b"),
+    },
+    {
+        "ago",
+        lnav::pcre2pp::code::from_const("\\Aago\\b"),
+    },
+    {
+        "lter",
+        lnav::pcre2pp::code::from_const("\\Alater\\b"),
+    },
+    {
+        "bfor",
+        lnav::pcre2pp::code::from_const("\\Abefore\\b"),
+    },
+    {
+        "aft",
+        lnav::pcre2pp::code::from_const("\\Aafter\\b"),
+    },
+    {
+        "now",
+        lnav::pcre2pp::code::from_const("\\Anow\\b"),
+    },
+    {
+        "here",
+        lnav::pcre2pp::code::from_const("\\Ahere\\b"),
+    },
+    {
+        "next",
+        lnav::pcre2pp::code::from_const("\\Anext\\b"),
+    },
+    {
+        "previous",
+        lnav::pcre2pp::code::from_const("\\A(?:previous\\b|last\\b)"),
+    },
 };
 
 static int64_t TIME_SCALES[] = {
-        1000 * 1000,
-        60,
-        60,
-        24,
+    1000 * 1000,
+    60,
+    60,
+    24,
 };
 
 const char relative_time::FIELD_CHARS[] = {
@@ -104,10 +219,8 @@ const char relative_time::FIELD_CHARS[] = {
 };
 
 Result<relative_time, relative_time::parse_error>
-relative_time::from_str(const char *str, size_t len)
+relative_time::from_str(string_fragment str)
 {
-    pcre_input pi(str, 0, len);
-    pcre_context_static<30> pc;
     int64_t number = 0;
     bool number_set = false, number_was_set = false;
     bool next_set = false;
@@ -117,13 +230,14 @@ relative_time::from_str(const char *str, size_t len)
     parse_error pe_out;
     std::unordered_set<int> seen_tokens;
 
-    pe_out.pe_column = -1;
+    pe_out.pe_column = 0;
     pe_out.pe_msg.clear();
 
+    auto remaining = str;
     while (true) {
         rt_field_type curr_field_type = RTF__MAX;
 
-        if (pi.pi_next_offset >= pi.pi_length) {
+        if (remaining.empty()) {
             if (number_set) {
                 if (number > 1970 && number < 2050) {
                     retval.rt_field[RTF_YEARS] = number - 1900;
@@ -132,17 +246,18 @@ relative_time::from_str(const char *str, size_t len)
                     switch (base_token) {
                         case RTT_BEFORE: {
                             auto epoch = retval.to_timeval();
-                            retval.rt_duration =
-                                std::chrono::duration_cast<std::chrono::microseconds>(
-                                    std::chrono::seconds(epoch.tv_sec)) +
-                                    std::chrono::microseconds(epoch.tv_usec);
+                            retval.rt_duration
+                                = std::chrono::duration_cast<
+                                      std::chrono::microseconds>(
+                                      std::chrono::seconds(epoch.tv_sec))
+                                + std::chrono::microseconds(epoch.tv_usec);
                             retval.rt_field[RTF_YEARS] = 70;
                             break;
                         }
                         case RTT_AFTER:
                             retval.rt_duration = std::chrono::duration_cast<
                                 std::chrono::microseconds>(
-                                    std::chrono::hours(24 * 365 * 200));
+                                std::chrono::hours(24 * 365 * 200));
                             break;
                         default:
                             break;
@@ -157,10 +272,14 @@ relative_time::from_str(const char *str, size_t len)
             if (base_token != RTT_INVALID) {
                 switch (base_token) {
                     case RTT_BEFORE:
-                        pe_out.pe_msg = "'before' requires a point in time (e.g. before 10am)";
+                        pe_out.pe_msg
+                            = "'before' requires a point in time (e.g. before "
+                              "10am)";
                         break;
                     case RTT_AFTER:
-                        pe_out.pe_msg = "'after' requires a point in time (e.g. after 10am)";
+                        pe_out.pe_msg
+                            = "'after' requires a point in time (e.g. after "
+                              "10am)";
                         break;
                     default:
                         ensure(false);
@@ -175,12 +294,20 @@ relative_time::from_str(const char *str, size_t len)
 
         bool found = false;
         for (int lpc = 0; lpc < RTT__MAX && !found; lpc++) {
+            static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+
             token_t token = (token_t) lpc;
-            if (!MATCHERS[lpc].pcre.match(pc, pi, PCRE_ANCHORED)) {
+            auto match_res = MATCHERS[lpc]
+                                 .pcre.capture_from(remaining)
+                                 .into(md)
+                                 .matches()
+                                 .ignore_error();
+            if (!match_res) {
                 continue;
             }
 
-            pe_out.pe_column = pc.all()->c_begin;
+            remaining = match_res->f_remaining;
+            pe_out.pe_column = match_res->f_all.sf_begin;
             found = true;
             if (RTT_MICROS <= token && token <= RTT_YEARS) {
                 if (!number_set) {
@@ -202,8 +329,8 @@ relative_time::from_str(const char *str, size_t len)
                 case RTT_TODAY:
                 case RTT_NOW: {
                     if (seen_tokens.count(token) > 0) {
-                        pe_out.pe_msg =
-                            "Current time reference has already been used";
+                        pe_out.pe_msg
+                            = "Current time reference has already been used";
                         return Err(pe_out);
                     }
 
@@ -227,7 +354,8 @@ relative_time::from_str(const char *str, size_t len)
                             retval.rt_field[RTF_HOURS] = tm.et_tm.tm_hour;
                             retval.rt_field[RTF_MINUTES] = tm.et_tm.tm_min;
                             retval.rt_field[RTF_SECONDS] = tm.et_tm.tm_sec;
-                            retval.rt_field[RTF_MICROSECONDS] = tm.et_nsec / 1000;
+                            retval.rt_field[RTF_MICROSECONDS]
+                                = tm.et_nsec / 1000;
                             break;
                         case RTT_YESTERDAY:
                             retval.rt_field[RTF_DAYS].value -= 1;
@@ -267,15 +395,15 @@ relative_time::from_str(const char *str, size_t len)
                         number_set = false;
                     }
                     if (!retval.is_absolute(RTF_YEARS)) {
-                        pe_out.pe_msg = "Expecting absolute time with A.M. or P.M.";
+                        pe_out.pe_msg
+                            = "Expecting absolute time with A.M. or P.M.";
                         return Err(pe_out);
                     }
                     if (token == RTT_AM) {
                         if (retval.rt_field[RTF_HOURS].value == 12) {
                             retval.rt_field[RTF_HOURS] = 0;
                         }
-                    }
-                    else if (retval.rt_field[RTF_HOURS].value < 12) {
+                    } else if (retval.rt_field[RTF_HOURS].value < 12) {
                         retval.rt_field[RTF_HOURS].value += 12;
                     }
                     if (base_token == RTT_AFTER) {
@@ -284,17 +412,20 @@ relative_time::from_str(const char *str, size_t len)
 
                         if (retval.rt_field[RTF_MICROSECONDS].value > 0) {
                             usecs += std::chrono::microseconds(
-                                1000000ULL - retval.rt_field[RTF_MICROSECONDS].value);
+                                1000000ULL
+                                - retval.rt_field[RTF_MICROSECONDS].value);
                             carry = 1;
                         }
                         if (carry || retval.rt_field[RTF_SECONDS].value > 0) {
                             usecs += std::chrono::seconds(
-                                60 - carry - retval.rt_field[RTF_SECONDS].value);
+                                60 - carry
+                                - retval.rt_field[RTF_SECONDS].value);
                             carry = 1;
                         }
                         if (carry || retval.rt_field[RTF_MINUTES].value > 0) {
                             usecs += std::chrono::minutes(
-                                60 - carry - retval.rt_field[RTF_MINUTES].value);
+                                60 - carry
+                                - retval.rt_field[RTF_MINUTES].value);
                             carry = 1;
                         }
                         usecs += std::chrono::hours(
@@ -302,11 +433,15 @@ relative_time::from_str(const char *str, size_t len)
                         retval.rt_duration = usecs;
                     }
                     if (base_token == RTT_BEFORE) {
-                        retval.rt_duration =
-                            std::chrono::hours(retval.rt_field[RTF_HOURS].value) +
-                            std::chrono::minutes(retval.rt_field[RTF_MINUTES].value) +
-                            std::chrono::seconds(retval.rt_field[RTF_SECONDS].value) +
-                            std::chrono::microseconds(retval.rt_field[RTF_MICROSECONDS].value);
+                        retval.rt_duration
+                            = std::chrono::hours(
+                                  retval.rt_field[RTF_HOURS].value)
+                            + std::chrono::minutes(
+                                  retval.rt_field[RTF_MINUTES].value)
+                            + std::chrono::seconds(
+                                  retval.rt_field[RTF_SECONDS].value)
+                            + std::chrono::microseconds(
+                                  retval.rt_field[RTF_MICROSECONDS].value);
                         retval.rt_field[RTF_HOURS].value = 0;
                         retval.rt_field[RTF_MINUTES].value = 0;
                         retval.rt_field[RTF_SECONDS].value = 0;
@@ -322,32 +457,31 @@ relative_time::from_str(const char *str, size_t len)
                 case RTT_AT:
                     break;
                 case RTT_TIME: {
-                    string hstr = pi.get_substr(pc[0]);
-                    string mstr = pi.get_substr(pc[1]);
+                    const auto hstr = md[1]->to_string();
+                    const auto mstr = md[2]->to_string();
                     retval.rt_field[RTF_HOURS] = atoi(hstr.c_str());
                     retval.rt_field[RTF_MINUTES] = atoi(mstr.c_str());
-                    if (pc[2]->is_valid()) {
-                        string sstr = pi.get_substr(pc[2]);
+                    if (md[3]) {
+                        const auto sstr = md[3]->to_string();
                         retval.rt_field[RTF_SECONDS] = atoi(sstr.c_str());
-                        if (pc[3]->is_valid()) {
-                            string substr = pi.get_substr(pc[3]);
+                        if (md[4]) {
+                            const auto substr = md[4]->to_string();
 
                             switch (substr.length()) {
                                 case 3:
-                                    retval.rt_field[RTF_MICROSECONDS] =
-                                        atoi(substr.c_str()) * 1000;
+                                    retval.rt_field[RTF_MICROSECONDS]
+                                        = atoi(substr.c_str()) * 1000;
                                     break;
                                 case 6:
-                                    retval.rt_field[RTF_MICROSECONDS] =
-                                        atoi(substr.c_str());
+                                    retval.rt_field[RTF_MICROSECONDS]
+                                        = atoi(substr.c_str());
                                     break;
                             }
                         } else {
                             retval.rt_field[RTF_MICROSECONDS].clear();
                             retval.rt_duration = 1s;
                         }
-                    }
-                    else {
+                    } else {
                         retval.rt_field[RTF_SECONDS].clear();
                         retval.rt_field[RTF_MICROSECONDS].clear();
                         retval.rt_duration = 1min;
@@ -357,16 +491,20 @@ relative_time::from_str(const char *str, size_t len)
                 }
                 case RTT_NUMBER: {
                     if (number_set) {
-                        pe_out.pe_msg = "No time unit given for the previous number";
+                        pe_out.pe_msg
+                            = "No time unit given for the previous number";
                         return Err(pe_out);
                     }
 
-                    string numstr = pi.get_substr(pc[0]);
+                    auto num_scan_res
+                        = scn::scan_value<int64_t>(md[0]->to_string_view());
 
-                    if (sscanf(numstr.c_str(), "%" PRId64, &number) != 1) {
-                        pe_out.pe_msg = "Invalid number: " + numstr;
+                    if (!num_scan_res) {
+                        pe_out.pe_msg = fmt::format(
+                            FMT_STRING("Invalid number: {}"), md[0].value());
                         return Err(pe_out);
                     }
+                    number = num_scan_res.value();
                     number_set = true;
                     break;
                 }
@@ -455,9 +593,12 @@ relative_time::from_str(const char *str, size_t len)
                     }
                     for (int field = 0; field < RTF__MAX; field++) {
                         if (retval.rt_field[field].value > 0) {
-                            retval.rt_field[field] = -retval.rt_field[field].value;
+                            retval.rt_field[field]
+                                = -retval.rt_field[field].value;
                         }
-                        if (last_field_type != RTF__MAX && field < last_field_type) {
+                        if (last_field_type != RTF__MAX
+                            && field < last_field_type)
+                        {
                             retval.rt_field[field] = 0;
                         }
                     }
@@ -468,7 +609,8 @@ relative_time::from_str(const char *str, size_t len)
                 case RTT_BEFORE:
                 case RTT_AFTER:
                     if (base_token != RTT_INVALID) {
-                        pe_out.pe_msg = "Before/after ranges are not supported yet";
+                        pe_out.pe_msg
+                            = "Before/after ranges are not supported yet";
                         return Err(pe_out);
                     }
                     base_token = token;
@@ -495,9 +637,8 @@ relative_time::from_str(const char *str, size_t len)
                 case RTT_NOON:
                     retval.rt_field[RTF_HOURS] = 12;
                     retval.rt_absolute_field_end = RTF__MAX;
-                    for (int lpc2 = RTF_MICROSECONDS;
-                         lpc2 < RTF_HOURS;
-                         lpc2++) {
+                    for (int lpc2 = RTF_MICROSECONDS; lpc2 < RTF_HOURS; lpc2++)
+                    {
                         retval.rt_field[lpc2] = 0;
                     }
                     break;
@@ -513,28 +654,30 @@ relative_time::from_str(const char *str, size_t len)
                         switch (base_token) {
                             case RTT_BEFORE:
                                 if (token == RTT_SUNDAY) {
-                                    pe_out.pe_msg =
-                                        "Sunday is the start of the week, so "
-                                        "there is nothing before it";
+                                    pe_out.pe_msg
+                                        = "Sunday is the start of the week, so "
+                                          "there is nothing before it";
                                     return Err(pe_out);
                                 }
-                                for (int wday = RTT_SUNDAY;
-                                     wday < token;
-                                     wday++) {
-                                    retval.rt_included_days.insert((token_t) wday);
+                                for (int wday = RTT_SUNDAY; wday < token;
+                                     wday++)
+                                {
+                                    retval.rt_included_days.insert(
+                                        (token_t) wday);
                                 }
                                 break;
                             case RTT_AFTER:
                                 if (token == RTT_SATURDAY) {
-                                    pe_out.pe_msg =
-                                        "Saturday is the end of the week, so "
-                                        "there is nothing after it";
+                                    pe_out.pe_msg
+                                        = "Saturday is the end of the week, so "
+                                          "there is nothing after it";
                                     return Err(pe_out);
                                 }
-                                for (int wday = RTT_SATURDAY;
-                                     wday > token;
-                                     wday--) {
-                                    retval.rt_included_days.insert((token_t) wday);
+                                for (int wday = RTT_SATURDAY; wday > token;
+                                     wday--)
+                                {
+                                    retval.rt_included_days.insert(
+                                        (token_t) wday);
                                 }
                                 break;
                             default:
@@ -551,13 +694,13 @@ relative_time::from_str(const char *str, size_t len)
                     break;
 
                 case RTT__MAX:
-                    assert(false);
+                    ensure(false);
                     break;
             }
 
-            if (token != RTT_NEXT &&
-                token != RTT_PREVIOUS &&
-                token != RTT_WHITE) {
+            if (token != RTT_NEXT && token != RTT_PREVIOUS
+                && token != RTT_WHITE)
+            {
                 next_set = false;
             }
 
@@ -574,7 +717,8 @@ relative_time::from_str(const char *str, size_t len)
     }
 }
 
-void relative_time::rollover()
+void
+relative_time::rollover()
 {
     for (int lpc = 0; lpc < RTF_DAYS; lpc++) {
         if (!this->rt_field[lpc].is_set) {
@@ -605,7 +749,8 @@ void relative_time::rollover()
     }
 }
 
-relative_time relative_time::from_timeval(const struct timeval& tv)
+relative_time
+relative_time::from_timeval(const struct timeval& tv)
 {
     relative_time retval;
 
@@ -617,7 +762,8 @@ relative_time relative_time::from_timeval(const struct timeval& tv)
     return retval;
 }
 
-relative_time relative_time::from_usecs(std::chrono::microseconds usecs)
+relative_time
+relative_time::from_usecs(std::chrono::microseconds usecs)
 {
     relative_time retval;
 
@@ -628,9 +774,10 @@ relative_time relative_time::from_usecs(std::chrono::microseconds usecs)
     return retval;
 }
 
-std::string relative_time::to_string() const
+std::string
+relative_time::to_string() const
 {
-    static const char *DAYS[] = {
+    static const char* DAYS[] = {
         "sun",
         "mon",
         "tue",
@@ -641,60 +788,77 @@ std::string relative_time::to_string() const
     };
 
     char dst[128] = "";
-    char *pos = dst;
+    char* pos = dst;
 
     if (this->is_absolute()) {
         for (const auto& day_token : this->rt_included_days) {
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
-                            "%s ", DAYS[day_token - RTT_SUNDAY]);
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
+                            "%s ",
+                            DAYS[day_token - RTT_SUNDAY]);
         }
 
-        pos += snprintf(pos, sizeof(dst) - (pos - dst),
-                        "%s",
-                        this->rt_next ? "next " :
-                        (this->rt_previous ? "last " : ""));
-        if (this->rt_field[RTF_YEARS].is_set &&
-            (this->rt_next || this->rt_previous ||
-             this->rt_field[RTF_YEARS].value != 0)) {
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
+        pos += snprintf(
+            pos,
+            sizeof(dst) - (pos - dst),
+            "%s",
+            this->rt_next ? "next " : (this->rt_previous ? "last " : ""));
+        if (this->rt_field[RTF_YEARS].is_set
+            && (this->rt_next || this->rt_previous
+                || this->rt_field[RTF_YEARS].value != 0))
+        {
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
                             "year %" PRId64 " ",
                             this->rt_field[RTF_YEARS].value);
-        } else if ((this->rt_next || this->rt_previous) &&
-                   this->rt_field[RTF_MONTHS].is_set) {
+        } else if ((this->rt_next || this->rt_previous)
+                   && this->rt_field[RTF_MONTHS].is_set)
+        {
             pos += snprintf(pos, sizeof(dst) - (pos - dst), "year ");
         }
-        if (this->rt_field[RTF_MONTHS].is_set &&
-            (this->rt_next || this->rt_previous ||
-             this->rt_field[RTF_MONTHS].value != 0)) {
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
+        if (this->rt_field[RTF_MONTHS].is_set
+            && (this->rt_next || this->rt_previous
+                || this->rt_field[RTF_MONTHS].value != 0))
+        {
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
                             "month %" PRId64 " ",
                             this->rt_field[RTF_MONTHS].value);
-        } else if ((this->rt_next || this->rt_previous) &&
-                   this->rt_field[RTF_DAYS].is_set) {
+        } else if ((this->rt_next || this->rt_previous)
+                   && this->rt_field[RTF_DAYS].is_set)
+        {
             pos += snprintf(pos, sizeof(dst) - (pos - dst), "month ");
         }
-        if (this->rt_field[RTF_DAYS].is_set &&
-            (this->rt_next || this->rt_previous ||
-             this->rt_field[RTF_DAYS].value != 0)) {
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
+        if (this->rt_field[RTF_DAYS].is_set
+            && (this->rt_next || this->rt_previous
+                || this->rt_field[RTF_DAYS].value != 0))
+        {
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
                             "day %" PRId64 " ",
                             this->rt_field[RTF_DAYS].value);
-        } else if ((this->rt_next || this->rt_previous) &&
-                   this->rt_field[RTF_HOURS].is_set) {
+        } else if ((this->rt_next || this->rt_previous)
+                   && this->rt_field[RTF_HOURS].is_set)
+        {
             pos += snprintf(pos, sizeof(dst) - (pos - dst), "day ");
         }
-        pos += snprintf(pos, sizeof(dst) - (pos - dst),
+        pos += snprintf(pos,
+                        sizeof(dst) - (pos - dst),
                         "%" PRId64 ":%02" PRId64,
                         this->rt_field[RTF_HOURS].value,
                         this->rt_field[RTF_MINUTES].value);
-        if (this->rt_field[RTF_SECONDS].is_set &&
-            this->rt_field[RTF_SECONDS].value != 0) {
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
+        if (this->rt_field[RTF_SECONDS].is_set
+            && this->rt_field[RTF_SECONDS].value != 0)
+        {
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
                             ":%.02" PRId64,
                             this->rt_field[RTF_SECONDS].value);
-            if (this->rt_field[RTF_MICROSECONDS].is_set &&
-                this->rt_field[RTF_MICROSECONDS].value != 0) {
-                pos += snprintf(pos, sizeof(dst) - (pos - dst),
+            if (this->rt_field[RTF_MICROSECONDS].is_set
+                && this->rt_field[RTF_MICROSECONDS].value != 0)
+            {
+                pos += snprintf(pos,
+                                sizeof(dst) - (pos - dst),
                                 ".%.03" PRId64,
                                 this->rt_field[RTF_MICROSECONDS].value / 1000);
             }
@@ -704,7 +868,8 @@ std::string relative_time::to_string() const
             if (this->rt_field[lpc].value == 0) {
                 continue;
             }
-            pos += snprintf(pos, sizeof(dst) - (pos - dst),
+            pos += snprintf(pos,
+                            sizeof(dst) - (pos - dst),
                             "%" PRId64 "%c",
                             this->rt_field[lpc].value,
                             FIELD_CHARS[lpc]);
@@ -720,98 +885,104 @@ std::string relative_time::to_string() const
     return dst;
 }
 
-struct exttm relative_time::adjust(const exttm &tm) const
+struct exttm
+relative_time::adjust(const exttm& tm) const
 {
     auto retval = tm;
 
-    if (this->rt_field[RTF_MICROSECONDS].is_set && this->is_absolute(RTF_MICROSECONDS)) {
+    if (this->rt_field[RTF_MICROSECONDS].is_set
+        && this->is_absolute(RTF_MICROSECONDS))
+    {
         retval.et_nsec = this->rt_field[RTF_MICROSECONDS].value * 1000;
-    }
-    else {
+    } else {
         retval.et_nsec += this->rt_field[RTF_MICROSECONDS].value * 1000;
     }
     if (this->rt_field[RTF_SECONDS].is_set && this->is_absolute(RTF_SECONDS)) {
-        if (this->rt_next &&
-            this->rt_field[RTF_SECONDS].value <= tm.et_tm.tm_sec) {
+        if (this->rt_next
+            && this->rt_field[RTF_SECONDS].value <= tm.et_tm.tm_sec)
+        {
             retval.et_tm.tm_min += 1;
         }
-        if (this->rt_previous &&
-            this->rt_field[RTF_SECONDS].value >= tm.et_tm.tm_sec) {
+        if (this->rt_previous
+            && this->rt_field[RTF_SECONDS].value >= tm.et_tm.tm_sec)
+        {
             retval.et_tm.tm_min -= 1;
         }
         retval.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_sec += this->rt_field[RTF_SECONDS].value;
     }
     if (this->rt_field[RTF_MINUTES].is_set && this->is_absolute(RTF_MINUTES)) {
-        if (this->rt_next &&
-            this->rt_field[RTF_MINUTES].value <= tm.et_tm.tm_min) {
+        if (this->rt_next
+            && this->rt_field[RTF_MINUTES].value <= tm.et_tm.tm_min)
+        {
             retval.et_tm.tm_hour += 1;
         }
-        if (this->rt_previous && (this->rt_field[RTF_MINUTES].value == 0 ||
-                                  (this->rt_field[RTF_MINUTES].value >= tm.et_tm.tm_min))) {
+        if (this->rt_previous
+            && (this->rt_field[RTF_MINUTES].value == 0
+                || (this->rt_field[RTF_MINUTES].value >= tm.et_tm.tm_min)))
+        {
             retval.et_tm.tm_hour -= 1;
         }
         retval.et_tm.tm_min = this->rt_field[RTF_MINUTES].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_min += this->rt_field[RTF_MINUTES].value;
     }
     if (this->rt_field[RTF_HOURS].is_set && this->is_absolute(RTF_HOURS)) {
-        if (this->rt_next &&
-            this->rt_field[RTF_HOURS].value <= tm.et_tm.tm_hour) {
+        if (this->rt_next
+            && this->rt_field[RTF_HOURS].value <= tm.et_tm.tm_hour)
+        {
             retval.et_tm.tm_mday += 1;
         }
-        if (this->rt_previous &&
-            this->rt_field[RTF_HOURS].value >= tm.et_tm.tm_hour) {
+        if (this->rt_previous
+            && this->rt_field[RTF_HOURS].value >= tm.et_tm.tm_hour)
+        {
             retval.et_tm.tm_mday -= 1;
         }
         retval.et_tm.tm_hour = this->rt_field[RTF_HOURS].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_hour += this->rt_field[RTF_HOURS].value;
     }
     if (this->rt_field[RTF_DAYS].is_set && this->is_absolute(RTF_DAYS)) {
-        if (this->rt_next &&
-            this->rt_field[RTF_DAYS].value <= tm.et_tm.tm_mday) {
+        if (this->rt_next && this->rt_field[RTF_DAYS].value <= tm.et_tm.tm_mday)
+        {
             retval.et_tm.tm_mon += 1;
         }
-        if (this->rt_previous &&
-            this->rt_field[RTF_DAYS].value >= tm.et_tm.tm_mday) {
+        if (this->rt_previous
+            && this->rt_field[RTF_DAYS].value >= tm.et_tm.tm_mday)
+        {
             retval.et_tm.tm_mon -= 1;
         }
         retval.et_tm.tm_mday = this->rt_field[RTF_DAYS].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_mday += this->rt_field[RTF_DAYS].value;
     }
     if (this->rt_field[RTF_MONTHS].is_set && this->is_absolute(RTF_MONTHS)) {
-        if (this->rt_next &&
-            this->rt_field[RTF_MONTHS].value <= tm.et_tm.tm_mon) {
+        if (this->rt_next
+            && this->rt_field[RTF_MONTHS].value <= tm.et_tm.tm_mon)
+        {
             retval.et_tm.tm_year += 1;
         }
-        if (this->rt_previous &&
-            this->rt_field[RTF_MONTHS].value >= tm.et_tm.tm_mon) {
+        if (this->rt_previous
+            && this->rt_field[RTF_MONTHS].value >= tm.et_tm.tm_mon)
+        {
             retval.et_tm.tm_year -= 1;
         }
         retval.et_tm.tm_mon = this->rt_field[RTF_MONTHS].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_mon += this->rt_field[RTF_MONTHS].value;
     }
     if (this->rt_field[RTF_YEARS].is_set && this->is_absolute(RTF_YEARS)) {
         retval.et_tm.tm_year = this->rt_field[RTF_YEARS].value;
-    }
-    else {
+    } else {
         retval.et_tm.tm_year += this->rt_field[RTF_YEARS].value;
     }
 
     return retval;
 }
 
-nonstd::optional<exttm> relative_time::window_start(
-    const struct exttm &tm) const
+nonstd::optional<exttm>
+relative_time::window_start(const struct exttm& tm) const
 {
     auto retval = tm;
 
@@ -863,7 +1034,8 @@ nonstd::optional<exttm> relative_time::window_start(
     }
 
     if (!this->rt_included_days.empty()) {
-        auto iter = this->rt_included_days.find((token_t) (RTT_SUNDAY + tm.et_tm.tm_wday));
+        auto iter = this->rt_included_days.find(
+            (token_t) (RTT_SUNDAY + tm.et_tm.tm_wday));
 
         if (iter == this->rt_included_days.end()) {
             return nonstd::nullopt;
@@ -914,7 +1086,8 @@ nonstd::optional<exttm> relative_time::window_start(
     auto tv = tm.to_timeval();
     auto start_time = retval.to_timeval();
     auto end_time = relative_time::from_usecs(this->rt_duration)
-        .adjust(retval).to_timeval();
+                        .adjust(retval)
+                        .to_timeval();
 
     if (tv < start_time || end_time < tv) {
         return nonstd::nullopt;
@@ -923,7 +1096,8 @@ nonstd::optional<exttm> relative_time::window_start(
     return retval;
 }
 
-int64_t relative_time::to_microseconds() const
+int64_t
+relative_time::to_microseconds() const
 {
     int64_t retval;
 
@@ -942,7 +1116,9 @@ int64_t relative_time::to_microseconds() const
         etm.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
 
         auto epoch_secs = std::chrono::seconds(tm2sec(&etm.et_tm));
-        retval = std::chrono::duration_cast<std::chrono::microseconds>(epoch_secs).count();
+        retval
+            = std::chrono::duration_cast<std::chrono::microseconds>(epoch_secs)
+                  .count();
         retval += this->rt_field[RTF_MICROSECONDS].value;
     } else {
         retval = this->rt_field[RTF_YEARS].value * 12;
