@@ -46,6 +46,20 @@
 #include "shlex.hh"
 #include "view_curses.hh"
 
+#if defined HAVE_NCURSESW_CURSES_H
+#    include <ncursesw/term.h>
+#elif defined HAVE_NCURSESW_H
+#    include <term.h>
+#elif defined HAVE_NCURSES_CURSES_H
+#    include <ncurses/term.h>
+#elif defined HAVE_NCURSES_H
+#    include <term.h>
+#elif defined HAVE_CURSES_H
+#    include <term.h>
+#else
+#    error "SysV or X/Open-compatible Curses header file required"
+#endif
+
 using namespace std::chrono_literals;
 
 const struct itimerval ui_periodic_timer::INTERVAL = {
@@ -732,6 +746,8 @@ view_colors::init_roles(const lnav_theme& lt,
         = this->to_attrs(lt, lt.lt_style_alt_text, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_HIDDEN)]
         = this->to_attrs(lt, lt.lt_style_hidden, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_CURSOR_LINE)]
+        = this->to_attrs(lt, lt.lt_style_cursor_line, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ADJUSTED_TIME)]
         = this->to_attrs(lt, lt.lt_style_adjusted_time, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SKEWED_TIME)]
@@ -1204,8 +1220,6 @@ lab_color::operator!=(const lab_color& rhs) const
     return !(rhs == *this);
 }
 
-#include <term.h>
-
 Result<screen_curses, std::string>
 screen_curses::create()
 {
